/*============================================================================

                        PVK40 & AVR4PVK Example : Blink

 Compiler/IDE : WinAVR

 Processor : ATMEGA64
 Clock : XT 14.74560 MHz

 Processor fuse bits:
  CKOPT: XT up to 16MHz, rail-to-rail
  SUT, CKSET: XT 3.0-8.0MHz, 16Kck + 65ms
  JTAGEN: Enabled
  M103C: Normal Mode

 Please set the DIP switches on your PVK40 board the following way:
 1) Switch off all DIP switches except:
 2) Switch on B3 LED on S11

============================================================================*/


/*
   Includes
*/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>



/* ==========================================================================
   User definitions
*/
typedef unsigned int clock_t;
void ledch(void);
clock_t clock(void);

clock_t systimer;                      /* systimer is incremented every 33ms */
clock_t now;
clock_t lap;
unsigned int period;

#define LED_DDR        DDRA
#define LED_PORT       PORTA
#define LED_PIN        PINA
#define LED_BIT        3



/* ==========================================================================
   TIMER1 interrupt
*/
ISR (TIMER1_OVF_vect) {
  systimer++;
}



/* ==========================================================================
   Read systimer
*/
clock_t clock(void) {
  clock_t temp;

  cli();
  temp = systimer;
  sei();
  return temp;
}



/* ==========================================================================
   Change LED
*/
void ledch(void) {
  if (LED_PIN & _BV(LED_BIT))   //if the LED is on
    LED_PORT &= ~_BV(LED_BIT);  // turn it off
  else
    LED_PORT |= _BV(LED_BIT);   // turn it on
}



/* ==========================================================================
   Main function
*/
int main(void) {
  LED_DDR |= _BV(LED_BIT);

  /* initialisation of Timer1 */
  TCCR1B = 0x2;
  TIMSK = _BV (TOIE1);
  period = 15;                           /* LED on and off time */
  sei();                                 //global interrupts enable
  /*--------------------------*/

  //main endless loop
  for(;;) {
    now = clock();
    if ((now - lap) > period) {
      lap += period;
      ledch();
    }
  }
}
