/*============================================================================

                        PVK40 & AVR4PVK Example : Display

 Compiler/IDE : WinAVR

 Processor : ATMEGA64
 Clock : XT 14.74560 MHz

 Processor fuse bits:
  CKOPT: XT up to 16MHz, rail-to-rail
  SUT, CKSET: XT 3.0-8.0MHz, 16Kck + 65ms
  JTAGEN: Enabled
  M103C: Normal Mode

 Please set the DIP switches on your PVK40 board the following way:
 1) Switch off all DIP switches except:
 2) Switch on B0,1,2,4 DISP on S11

============================================================================*/


/*
   Includes
*/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>



/* ==========================================================================
   User definitions
*/
void delay(unsigned char value);

char segment[8] = {0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80};    /* 12345678 */

#define BUTTONS_DDR     DDRC
#define BUTTONS_PIN     PINC
#define BUTTONS_PORT    PORTC

#define DISP1_DDR     DDRA
#define DISP1_PORT    PORTA
#define DISP1_BIT     0



/* ==========================================================================
   Delay
*/
void delay(unsigned char value) {
  while(value--)  //while value>0, with decrement
    asm("nop");   // do assembler NOP instruction
}



/* ==========================================================================
   Main function
*/
int main(void) {
  unsigned char i;
  unsigned char temp;

  BUTTONS_DDR = 0xFF;
  DISP1_DDR |= _BV(DISP1_BIT);

  for(;;) {
    DISP1_PORT |= _BV(DISP1_BIT);
    BUTTONS_DDR = 0x00;                               /* PORTD is input */
    delay(100);                                       /* we must wait before reading the input to get correct value */
    temp = BUTTONS_PIN;
    BUTTONS_DDR = 0xFF;                               /* PORTD is output */

    for(i=0; i<8; i++) {
      if (!(temp & 1)) {
        BUTTONS_PORT = segment[i];                    /* show i value as the most right character on the display */
        DISP1_PORT &= ~_BV(DISP1_BIT);
        break;
      }
      temp >>= 1;
    }
    delay(250);
  }
}
