/*============================================================================

                        PVK40 & AVR4PVK Example : Counter

 Compiler/IDE : WinAVR

 Processor : ATMEGA64
 Clock : XT 14.74560 MHz

 Processor fuse bits:
  CKOPT: XT up to 16MHz, rail-to-rail
  SUT, CKSET: XT 3.0-8.0MHz, 16Kck + 65ms
  JTAGEN: Enabled
  M103C: Normal Mode

 Please set the DIP switches on your PVK40 board the following way:
 1) Switch off all DIP switches except:
 2) Switch on B0,1,2,4 DISP on S11

============================================================================*/


/*
   Includes
*/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>



/* ==========================================================================
   User definitions
*/
typedef unsigned int clock_t;
char digit[5] = {0xFF,0xFE,0xFD,0xFB,0xEF};
char segment[10] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90};

clock_t systimer;                           /* systimer is incremented every 30ms */
clock_t now;
clock_t lap;
unsigned int period;
unsigned int count = 0;

#define BUTTONS_DDR     DDRC
#define BUTTONS_PIN     PINC
#define BUTTONS_PORT    PORTC

#define DISP_DDR        DDRA
#define DISP_PORT       PORTA
#define DISP1_BIT       0
#define DISP2_BIT       1
#define DISP3_BIT       2
#define DISP4_BIT       4

void delay(unsigned int value);
clock_t clock(void);
void dispdigit(int j, int p);



/* ==========================================================================
   Delay
*/
void delay(unsigned int value) {
  while(value--)
    asm("nop");
}



/* ==========================================================================
   TIMER1 interrupt
*/
ISR (TIMER1_OVF_vect) {
  systimer++;
}



/* ==========================================================================
   Read systimer
*/
clock_t clock(void) {
  clock_t temp;

  cli();
  temp = systimer;
  sei();
  return temp;
}



/* ==========================================================================
   dispdigit(j,p)
    shows the value of the integer j
    on one p position of the 7 segments display
*/
void dispdigit(int j, int p) {
  DISP_PORT = digit[0];          //all display segments are off
  BUTTONS_PORT = segment[j];   //here we write what character we want to show
  BUTTONS_DDR = 0xFF;            // buttons port is output, we use it for display
  DISP_PORT = digit[p];        //here we write what position of the display it will be shown on 
  delay(1000);
  BUTTONS_DDR = 0x00;            //buttons port is input, we use it for buttons
}



/* ==========================================================================
   Main function
*/
int main(void) {
  DISP_DDR |= _BV(DISP1_BIT) | _BV(DISP2_BIT) | _BV(DISP3_BIT) | _BV(DISP4_BIT); // set the pins controlling the display anodes to be output

  /* initialisation of Timer1 */
  TCCR1B = 0x2;
  TIMSK = _BV (TOIE1);
  period=30;                                /* counter refresh time - 1 second*/
  sei();                                    //global interrupts enable
  /*--------------------------*/

 //endless program loop
  for(;;) {
    now = clock();
    if ((now - lap) > period) {
      lap += period;
      if(++count > 9999)                    /* counting */
        count = 0;
    }

    dispdigit(count%10, 1);
    dispdigit((count/10)%10, 2);
    dispdigit((count/100)%10, 3);
    dispdigit(count/1000, 4);
  }
}
