/*============================================================================

                        PVK40 & AVR4PVK Example : UART - Tx

 Compiler/IDE : WinAVR

 Processor : ATMEGA64
 Clock : XT 14.74560 MHz

 Processor fuse bits:
  CKOPT: XT up to 16MHz, rail-to-rail
  SUT, CKSET: XT 3.0-8.0MHz, 16Kck + 65ms
  JTAGEN: Enabled
  M103C: Normal Mode

 Please set the DIP switches on your PVK40 board the following way:
 1) Switch off all DIP switches except:
 2) Switch on C6 UART_TX on S11
 3) If USB selected, insert UMS1 module into socket

============================================================================*/


/*
   Includes
*/
#include <inttypes.h>
#include <avr/io.h>



/* ==========================================================================
   User definitions
*/
void uart_init (void);



/* ==========================================================================
   serial port initialisation
*/
void uart_init (void) {
  UBRR1H = 0;
  UBRR1L = 95;                                        /* 9600Bd */
  UCSR1B = _BV(RXEN1);                                /* enable transmitter */
  UCSR1C = _BV(UCSZ10) | _BV(UCSZ11);                 /* Set frame format: 8 data, 1 stop bit */
}


{  /* serial port initialisation */
  RCSTA=0x80; /* pin RC6 is automaticaly configured as output */
  SPBRG=0x14; /* 9600Bd */
  TXSTA=0x26; /* TX enabled */


/* ==========================================================================
   send one byte
*/
void uart_sendbyte(unsigned char databyte) {
  while ( !( UCSR1A & (1<<UDRE1)) );                  /* wait for operation completion */
  UDR1 = databyte;
}



/* ==========================================================================
   send string
*/
void uart_sendstr(const char *s) {
  while(*s) {
    uart_sendbyte(*(s++));
  }
}



/* ==========================================================================
   Main function
*/
int main(void) {
  uart_init();

  uart_sendstr("Hello, World!\r\n");   //send this string

  for(;;);                             // do nothing
}
