/*============================================================================

                        PVK40 & AVR4PVK Example : UART - Rx

 Compiler/IDE : WinAVR

 Processor : ATMEGA64
 Clock : XT 14.74560 MHz

 Processor fuse bits:
  CKOPT: XT up to 16MHz, rail-to-rail
  SUT, CKSET: XT 3.0-8.0MHz, 16Kck + 65ms
  JTAGEN: Enabled
  M103C: Normal Mode

 Please set the DIP switches on your PVK40 board the following way:
 1) Switch off all DIP switches except:
 2) Switch on C7 UART_RX on S11
 3) Switch on B0,1,2,4 DISP on S11
 4) Select COM or USB on JP4
 5) If USB selected, insert UMS1 module into socket

============================================================================*/


/*
   Includes
*/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>



/* ==========================================================================
   User definitions
*/

void delay(unsigned int value);
void uart_init (void);
void display_init (void);
void dispdigit(int j, int p);

unsigned char buffer;

char digit[5] = {0xFF,0xFE,0xFD,0xFB,0xEF};
char segment[16] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90,
                    0x88,0x83,0xC6,0xA1,0x86,0x8E};

#define BUTTONS_DDR     DDRC
#define BUTTONS_PIN     PINC
#define BUTTONS_PORT    PORTC

#define DISP_DDR        DDRA
#define DISP_PORT       PORTA
#define DISP1_BIT       0
#define DISP2_BIT       1
#define DISP3_BIT       2
#define DISP4_BIT       4



/* ==========================================================================
   Delay
*/
void delay(unsigned int value) {
  while(value--)
    asm("nop");
}



/* ==========================================================================
   USART1_RXC interrupt
*/
ISR (USART1_RX_vect) {
  buffer = UDR1;
}



/* ==========================================================================
   serial port initialization
*/
void uart_init (void) {
  UBRR1H = 0;
  UBRR1L = 95;                                        /* 9600Bd */
  UCSR1B = _BV(RXCIE1) | _BV(RXEN1);                  /* enable transmitter and interrupt */
  UCSR1C = _BV(UCSZ10) | _BV(UCSZ11);                 /* Set frame format: 8 data, 1 stop bit */
}



/* ==========================================================================
   display initialization
*/
void display_init (void) {
  DISP_DDR |= _BV(DISP1_BIT) | _BV(DISP2_BIT) | _BV(DISP3_BIT) | _BV(DISP4_BIT);
}



/* ==========================================================================
   dispdigit(j,p)
    shows the value of an integer j
    on one p position of the 7 segments display
*/
void dispdigit(int j, int p) {
  DISP_PORT = digit[0];
  BUTTONS_PORT = segment[j];
  BUTTONS_DDR = 0xFF;
  DISP_PORT = digit[p];
  delay(1000);
  BUTTONS_DDR = 0x00;
}



/* ==========================================================================
   Main function
*/
int main(void) {
  unsigned char mbuffer;

  uart_init();
  display_init();

  while ( !(UCSR1A & (1<<RXC1)) )                     /* wait for first byte */
  sei();                                              /* interrupts are enabled */

  for(;;) {
    cli();                                            /* interrupts are disabled */
    mbuffer = buffer;
    sei();                                            /* interrupts are enabled */
    dispdigit(mbuffer & 0x0F, 1);                     //show low half-byte of the received sign
    dispdigit(mbuffer >> 4, 2);                       //show high half-byte of the received sign
  }
}
